@@ -1,7 +1,8 @@
-{-# LANGUAGE FlexibleInstances, MultiParamTypeClasses, TypeSynonymInstances #-}
+{-# LANGUAGE FlexibleInstances, MultiParamTypeClasses #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Layout.Column
+-- Description :  Layout that places all windows in one column.
 -- Copyright   :  (c) 2009 Ilya Portnov
 -- License     :  BSD3-style (see LICENSE)
 --
@@ -40,7 +41,7 @@ import qualified XMonad.StackSet as W
 -- In this example, each next window will have height 1.6 times less then
 -- previous window.
 
-data Column a = Column Float deriving (Read,Show)
+newtype Column a = Column Float deriving (Read,Show)
 
 instance LayoutClass Column a where
     pureLayout = columnLayout
@@ -57,15 +58,13 @@ columnLayout (Column q) rect stack = zip ws rects
           n = length ws
           heights = map (xn n rect q) [1..n]
           ys = [fromIntegral $ sum $ take k heights | k <- [0..n-1]]
-          rects = map (mkRect rect) $ zip heights ys
+          rects = zipWith (curry (mkRect rect)) heights ys
 
 mkRect :: Rectangle -> (Dimension,Position) -> Rectangle
 mkRect (Rectangle xs ys ws _) (h,y) = Rectangle xs (ys+fromIntegral y) ws h
 
 xn :: Int -> Rectangle -> Float -> Int -> Dimension
 xn n (Rectangle _ _ _ h) q k = if q==1 then
-                                  h `div` (fromIntegral n)
+                                  h `div` fromIntegral n
                                else
-                                  round ((fromIntegral h)*q^(n-k)*(1-q)/(1-q^n))
-
-
+                                  round (fromIntegral h*q^(n-k)*(1-q)/(1-q^n))
