@@ -30,15 +30,16 @@ module XMonad.Layout.IM (
 ) where
 
 import XMonad
-import qualified XMonad.StackSet as S
 import XMonad.Layout.Grid
 import XMonad.Layout.LayoutModifier
+import XMonad.Prelude
 import XMonad.Util.WindowProperties
+import qualified XMonad.StackSet as S
 
 import Control.Arrow (first)
 
 -- $usage
--- You can use this module with the following in your @~\/.xmonad\/xmonad.hs@:
+-- You can use this module with the following in your @xmonad.hs@:
 --
 -- > import XMonad.Layout.IM
 -- > import Data.Ratio ((%))
@@ -55,9 +56,9 @@ import Control.Arrow (first)
 --
 -- Screenshot: <http://haskell.org/haskellwiki/Image:Xmonad-layout-im.png>
 --
--- For more detailed instructions on editing the layoutHook see:
---
--- "XMonad.Doc.Extending#Editing_the_layout_hook"
+-- For more detailed instructions on editing the layoutHook see
+-- <https://xmonad.org/TUTORIAL.html#customizing-xmonad the tutorial> and
+-- "XMonad.Doc.Extending#Editing_the_layout_hook".
 
 -- $hints
 --
@@ -110,11 +111,6 @@ applyIM ratio prop wksp rect = do
             return (first ((w, masterRect) :) wrs)
         Nothing -> runLayout wksp rect
 
--- | Like find, but works with monadic computation instead of pure function.
-findM :: Monad m => (a -> m Bool) -> [a] -> m (Maybe a)
-findM _ [] = return Nothing
-findM f (x:xs) = do { b <- f x; if b then return (Just x) else findM f xs }
-
 -- | This is for compatibility with old configs only and will be removed in future versions!
 data IM a = IM Rational Property deriving (Read, Show)
 instance LayoutClass IM Window where
