@@ -1,6 +1,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Layout.IfMax
+-- Description :  Decide upon a layout depending on the number of windows.
 -- Copyright   :  (c) 2013 Ilya Portnov
 -- License     :  BSD3-style (see LICENSE)
 --
@@ -8,7 +9,7 @@
 -- Stability   :  unstable
 -- Portability :  unportable
 --
--- Provides IfMax layout, which will run one layout if there are maximum N 
+-- Provides IfMax layout, which will run one layout if there are maximum N
 -- windows on workspace, and another layout, when number of windows is greater
 -- than N.
 --
@@ -23,17 +24,16 @@ module XMonad.Layout.IfMax
     , ifMax
     ) where
 
-import Control.Applicative((<$>))
-import Control.Arrow
+import Control.Arrow ((&&&))
 import qualified Data.List as L
 import qualified Data.Map  as M
-import Data.Maybe
 
 import XMonad
+import XMonad.Prelude
 import qualified XMonad.StackSet as W
 
 -- $usage
--- IfMax layout will run one layout if number of windows on workspace is as 
+-- IfMax layout will run one layout if number of windows on workspace is as
 -- maximum N, and else will run another layout.
 --
 -- You can use this module by adding folowing in your @xmonad.hs@:
@@ -91,5 +91,4 @@ ifMax :: (LayoutClass l1 w, LayoutClass l2 w)
       -> l1 w           -- ^ First layout
       -> l2 w           -- ^ Second layout
       -> IfMax l1 l2 w
-ifMax n l1 l2 = IfMax n l1 l2
-
+ifMax = IfMax
