@@ -1,7 +1,7 @@
-{-# LANGUAGE DeriveDataTypeable #-}
-----------------------------------------------------------------------------
+-----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Util.Minimize
+-- Description :  Common utilities for window minimizing\/maximizing.
 -- Copyright   :  (c) Bogdan Sinitsyn (2016)
 -- License     :  BSD3-style (see LICENSE)
 --
@@ -28,7 +28,7 @@ data Minimized = Minimized
     { rectMap :: RectMap
     , minimizedStack :: [Window]
     }
-    deriving (Eq, Typeable, Read, Show)
+    deriving (Eq, Read, Show)
 
 instance ExtensionClass Minimized where
   initialValue = Minimized { rectMap = M.empty
